#include "TemplateElements.h"

#include "StringUtils.h"
#include "TemplateSubstitutionContext.h"

#include <iostream>

TemplateExtends::TemplateExtends(TemplateNode* parent, const String& path)
    : TemplateNode(parent)
{
    mPath = StringUtils::stripQuotes(path);
};

String TemplateExtends::getRawContent() const
{
    return "TemplateExtends: " + mPath;
}

String TemplateExtends::getPath() const
{
    return mPath;
}

TemplateBlock::TemplateBlock(TemplateNode* parent, const String& name)
    : TemplateNode(parent),
    mName(name)
{

}

void TemplateBlock::addLine(const String& line)
{
    mBody.push_back(line);
}

String TemplateBlock::getRawContent() const
{
    String content = "TemplateBlock: " + mName + " - Start \n";
    content += TemplateNode::getRawContent();
    content += "TemplateBlock: " + mName + " - End";
    return content;
}

String  TemplateBlock::getIdentifier() const
{
    return mName;
}

String TemplateBlock::renderAsParent(TemplateSubstitutionContext* substitutions, TemplateBlock* base)
{
    String content;
    for (auto& child : mChildren)
    {
        if (child->getType() == Type::EXPRESSION)
        {
            auto expression = dynamic_cast<TemplateExpression*>(child.get());
            if (expression->getContent() == "super()")
            {
                content += base->render(substitutions, nullptr);
            }
        }
        else if(child->getType() == Type::TEXT_BODY)
        {
            content += child->render(substitutions, nullptr);
        }
    }
    return content;
}

String TemplateBlock::renderAsLeaf(TemplateSubstitutionContext* substitutions)
{
    String content;
    for (auto& child : mChildren)
    {
        if(child->getType() == Type::TEXT_BODY)
        {
            content += child->render(substitutions, nullptr);
        }
    }
    return content;
}

String TemplateBlock::render(TemplateSubstitutionContext* substitutions, TemplateNode* parentContext)
{
    String content;
    if (parentContext)
    {
        if (auto parent_node = parentContext->getFirstChildShallow(Type::BLOCK, getIdentifier()))
        {
            content = dynamic_cast<TemplateBlock*>(parent_node)->renderAsParent(substitutions, this);
        }
        else
        {
            content= renderAsLeaf(substitutions);
        }
    }
    else
    {
        content= renderAsLeaf(substitutions);
    }
    return content;
}

TemplateExpression::TemplateExpression(TemplateNode* parent, const String& content)
    : TemplateNode(parent),
    mContent(content)
{

}

const String& TemplateExpression::getContent() const
{
    return mContent;
}

String TemplateExpression::render(TemplateSubstitutionContext* substitutions, TemplateNode*)
{
    if (substitutions && substitutions->hasSubstitution(mContent))
    {
        return substitutions->getSubstitution(mContent);
    }
    return {};
}

String TemplateExpression::getRawContent() const
{
    return "TemplateExpression: " + mContent;
}

TemplateTextBody::TemplateTextBody(TemplateNode* parent)
        : TemplateNode(parent)
{

}

void TemplateTextBody::addLine(const String& content)
{
    mContent.push_back(content);
}

bool TemplateTextBody::hasContent() const
{
    return !mContent.empty();
}

String TemplateTextBody::render(TemplateSubstitutionContext*, TemplateNode*)
{
    String content;
    for(unsigned idx=0; idx<mContent.size(); idx++)
    {
        content += mContent[idx];
        if(idx != mContent.size() - 1)
        {
            content += '\n';
        }
    }
    return content;
}

String TemplateTextBody::getRawContent() const
{
    String content;
    for(unsigned idx=0; idx<mContent.size(); idx++)
    {
        content += "Template Body L-" + std::to_string(idx) + ": " + mContent[idx];
        if(idx != mContent.size() - 1)
        {
            content += '\n';
        }
    }
    return content;
}
