#include "Bits.h"

bool Bits::MostSignificantBitIsOne(Byte c)
{
    return c & (1 << 7);
}

Word Bits::GetWordFirstBit(const Word word)
{
    return word & Bits::WORD_FIRST_BIT;
};

Word Bits::GetWordLastByte(const Word word)
{
    return word & Bits::WORD_LAST_BYTE;
}

unsigned char Bits::getHigherNBits(Byte input, size_t num)
{
    return input >> (8 - num);
}

unsigned char Bits::getByteN(DWord input, size_t n)
{
    return (input << 8*n) >> 24;
}

uint32_t Bits::mirror(DWord byte, size_t length)
{
    uint32_t ret{0};
    for(unsigned idx=0; idx<length; idx++)
    {
        if (getBitN(byte, length - 1 - idx))
        {
            ret |= (0x01 << idx);
        }
    }
    return ret;
}

unsigned char Bits::getLowerNBits(DWord input, size_t num)
{
    switch (num)
    {
    case 1:
        return input & 0x01;
    case 2:
        return input & 0x03;
    case 3:
        return input & 0x07;
    case 4:
        return input & 0x0F;
    case 5:
        return input & 0x1F;
    case 6:
        return input & 0x3F;
    case 7:
        return input & 0x7F;
    case 8:
        return static_cast<unsigned char>(input);
    default:
        return 0;
    }
}

unsigned char Bits::getTwoBitsAtN(Byte input, size_t n)
{
    return (input & (0x03 << n)) >> n;
}

unsigned char Bits::getMBitsAtN(Byte input, size_t m, size_t n)
{
    switch (m)
    {
    case 1:
        return (input & (0x01 << n)) >> n;
    case 2:
        return (input & (0x03 << n)) >> n;
    case 3:
        return (input & (0x07 << n)) >> n;
    case 4:
        return (input & (0x0F << n)) >> n;
    case 5:
        return (input & (0x1F << n)) >> n;
    case 6:
        return (input & (0x3F << n)) >> n;
    case 7:
        return (input & (0x7F << n)) >> n;
    case 8:
        return input;
    default:
        return 0;
    }
}

bool Bits::getBitN(DWord input, size_t n)
{
    return input & (1 << n);
}

unsigned char Bits::getFromString(const String& string)
{
    unsigned char ret{0};

    if (string.size() < 8)
    {
        return ret;
    }

    for(unsigned idx=0; idx<8; idx++)
    {
        if (string[idx] == '1')
        {
            ret |= (0x01 << (7 - idx));
        }
    }
    return ret;
}

String Bits::toString(DWord input, size_t length)
{
    String ret;
    if (length > 8)
    {
        unsigned overshoot = length - 8;
        for(unsigned idx=0; idx<overshoot; idx++)
        {
            ret += getBitN(input, length - 1 - idx) ? '1' : '0';
        }
        ret += "-";
        for(unsigned idx=0; idx<8; idx++)
        {
            ret += getBitN(input, 7 - idx) ? '1' : '0';
        }
    }
    else
    {
        for(unsigned idx=0; idx<length; idx++)
        {
            ret += getBitN(input, 7 - idx) ? '1' : '0';
        }
    }
    return ret;
}

void Bits::ReverseBuffer(Byte* buffer, char* reverse, size_t size, size_t targetSize)
{
    for(unsigned idx=0; idx<targetSize; idx++)
    {
        if (idx < size)
        {
            reverse[idx] = buffer[size - 1 -idx];
        }
        else
        {
            reverse[idx] = 0;
        }
    }
}

Word Bits::ToWord(Byte* buffer, bool reverse)
{
    return ToType<Word>(buffer, reverse);
}

DWord Bits::ToDWord(Byte* buffer, bool reverse)
{
    return ToType<DWord>(buffer, reverse);
}

QWord Bits::ToQWord(Byte* buffer, bool reverse)
{
    return ToType<QWord>(buffer, reverse);
}

bool Bits::Compare(Byte* buffer, const char* tag, size_t size)
{
    for(unsigned idx=0; idx<size; idx++)
    {
        if(tag[idx] != buffer[idx])
        {
            return false;
        }
    }
    return true;
}

bool Bits::CompareDWords(Byte* buffer, DWord target)
{
    for(size_t idx=0; idx<4;idx++)
    {
        if (buffer[idx] != getByteN(target, idx))
        {
            return false;
        }
    }
    return true;
}

bool Bits::CompareWords(Byte* buffer, Word target)
{
    bool first_byte_matches = (buffer[0] == GetWordFirstBit(target));
    if (!first_byte_matches)
    {
        return false;
    }
    return buffer[1] == GetWordLastByte(target);
}
