#include "BinaryStream.h"

#include "Bits.h"

Optional<Byte> BinaryStream::getNextByte(InputStream<Byte>& stream)
{
    return stream.get();
}

bool BinaryStream::getNextNBytes(InputStream<Byte>& stream, Byte* buffer, size_t number)
{
    Byte c{0};
    for(size_t idx=0; idx<number; idx++)
    {
        if(stream.get(c))
        {
            buffer[idx] = c;
        }
        else
        {
            return false;
        }
    }
    return true;
}

bool BinaryStream::getNextWord(InputStream<Byte>& stream, Byte* buffer)
{
    return getNextNBytes(stream, buffer, sizeof(Word));
}

bool BinaryStream::getNextDWord(InputStream<Byte>& stream, Byte* buffer)
{
    return getNextNBytes(stream, buffer, sizeof(DWord));
}

bool BinaryStream::getNextQWord(InputStream<Byte>& stream, Byte* buffer)
{
    return getNextNBytes(stream, buffer, sizeof(QWord));
}

Optional<Word> BinaryStream::getNextWord(InputStream<Byte>& stream, bool reverse)
{
    Byte buffer[sizeof(Word)];
    if(!getNextWord(stream, buffer))
    {
        return false;
    }
    return Bits::ToWord(buffer, reverse);
}

Optional<DWord> BinaryStream::getNextDWord(InputStream<Byte>& stream)
{
    Byte buffer[sizeof(DWord)];
    if(!BinaryStream::getNextDWord(stream, buffer))
    {
        return false;
    }
    return Bits::ToDWord(buffer);
}

Optional<QWord> BinaryStream::getNextQWord(InputStream<Byte>& stream)
{
    Byte buffer[sizeof(QWord)];
    if(!getNextQWord(stream, buffer))
    {
        return false;
    }
    return Bits::ToQWord(buffer);
}

bool BinaryStream::checkNextDWord(InputStream<Byte>& stream, DWord target)
{
    Byte buffer[sizeof(DWord)];
    if(!getNextDWord(stream, buffer))
    {
        return false;
    }

    if(!Bits::CompareDWords(buffer, target))
    {
        return false;
    }
    return true;
}

bool BinaryStream::getNextString(InputStream<Byte>& stream, String& target, size_t numBytes)
{
    Byte c{0};
    for(size_t idx=0; idx<numBytes; idx++)
    {
        if(!stream.get(c))
        {
            return false;
        }
        target += c;
    }
    return true;
}
