#pragma once

#include "AbstractChecksumCalculator.h"
#include "ByteTypes.h"
#include "String.h"
#include "Optional.h"
#include "Pair.h"

class BitStream
{
public:
    virtual ~BitStream();

    Byte getCurrentByte();

    int getCurrentByteOffset() const;

    size_t getCurrentBitOffset() const;

    virtual bool isFinished() const = 0;

    String logNextNBytes(size_t n) const;

    String logLocation();

    virtual void peekNextNBytes(size_t n, VecBytes& bytes) const = 0;

    virtual bool readNextNBits(size_t n, Byte& buffer);

    virtual Optional<Byte> readNextByte() = 0;

    virtual void writeNBits(DWord data, size_t length);

    virtual void writeByte(Byte data, bool checkOverflow = true) = 0;

    void write(DWord data);

    void writeWord(Word data);

    virtual void writeBytes(const VecBytes& data) = 0;

    void resetOffsets();

    virtual void reset();

    void flushRemainingBits();

    Pair<Byte, size_t> getRemainingBits() const;

    void setChecksumCalculator(AbstractChecksumCalculator* calc);

    void clearChecksumCalculator();

protected:
    int mByteOffset{-1};
    size_t mBitOffset{0};
    Byte mCurrentByte{0};

    int mEndByteOffset{-1};
    size_t mEndBitOffset{0};
    Byte mEndByte{0};
    AbstractChecksumCalculator* mChecksumCalculator{nullptr};
};
