#include "BufferBitStream.h"

#include "Bits.h"

bool BufferBitStream::isFinished() const
{
    return mByteOffset == static_cast<int>(mBuffer.size()) - 1;
}

void BufferBitStream::peekNextNBytes(size_t n, VecBytes& ret) const
{
    ret.resize(n, 0);
    size_t count = 0;

    int start = mByteOffset;
    if (start<0)
    {
        start = 0;
    }

    for(unsigned idx=start; idx<start + n; idx++)
    {
        if (idx == mBuffer.size())
        {
            break;
        }

        ret[count] = mBuffer[idx];
        count++;
    }
}

Optional<uint8_t> BufferBitStream::readNextByte()
{
    if (mByteOffset + 1 == static_cast<int>(mBuffer.size()))
    {
        return {};
    }
    else
    {
        mByteOffset++;
        mCurrentByte = mBuffer[mByteOffset];
        return mCurrentByte;
    }
}

void BufferBitStream::setBuffer(const VecBytes& data)
{
    mBuffer = data;
}

void BufferBitStream::writeByte(uint8_t data, bool checkOverflow)
{
    uint8_t out_byte{0};
    if (checkOverflow && mBitOffset > 0)
    {
        out_byte = Bits::getLowerNBits(mCurrentByte, mBitOffset);
        out_byte |= data << mBitOffset;

        mCurrentByte = Bits::getHigherNBits(data, mBitOffset);
    }
    else
    {
        out_byte = data;
    }

    if (mChecksumCalculator)
    {
        mChecksumCalculator->addValue(out_byte);
    }
    //std::cout << "Writing byte " << Bits::toString(out_byte) << " had bitoffset of " << mBitOffset << std::endl;
    mBuffer.push_back(out_byte);
}


void BufferBitStream::writeBytes(const VecBytes& data)
{
    mBuffer = data;
}

const VecBytes& BufferBitStream::getBuffer() const
{
    return mBuffer;
}

void BufferBitStream::reset()
{
    BitStream::reset();
    mBuffer.clear();
}

