#include "PointCollection.h"

template<size_t DIM>
PointCollection<DIM>::PointCollection(const Vector<Point<DIM> > points)
    : mPoints(points)
{

}

template<size_t DIM>
void PointCollection<DIM>::apply(const Transform& transform)
{
    for (auto& point : mPoints)
    {
        point.apply(transform);
    }
}

template<size_t DIM>
void PointCollection<DIM>::addPoint(const Point<DIM>& point)
{
    mPoints.push_back(point);
}

template<size_t DIM>
Bounds PointCollection<DIM>::getBounds() const
{
	Bounds bounds{0.0, 0.0, 0.0, 0.0};

    if (mPoints.size() == 0)
    {
        return bounds;
    }

	bounds.intialize(mPoints[0].getX(), mPoints[0].getY(), mPoints[0].getZ());
	for(const auto& point : mPoints)
	{
        bounds.includePoint(point.getX(), point.getY(), point.getZ());
	}
	return bounds;
}

template<size_t DIM>
Point<DIM> PointCollection<DIM>::getEndPoint() const
{
    if (mPoints.empty())
    {
        return {};
    }
    else
    {
        return mPoints[mPoints.size() - 1];
    }
}

template<size_t DIM>
const Vector<Point<DIM> >& PointCollection<DIM>::getPoints() const
{
    return mPoints;
}

template class PointCollection<2>;
template class PointCollection<3>;