#include "PngWriter.h"

#ifdef _WIN32
#include "Win32WicImageWriter.h"
#else
#include "BasicPngWriter.h"
#endif

PngWriter::PngWriter()
    : IImageWriter(ImgFormat::PNG)
{
#ifdef _WIN32
    mWriterImpl = std::make_unique<Win32WicImageWriter>(mFormat);
#else
    mWriterImpl = std::make_unique<BasicPngWriter>();
#endif
}

PngWriter::~PngWriter()
{

}

Ptr<PngWriter> PngWriter::Create()
{
    return std::make_unique<PngWriter>();
}

void PngWriter::setCompressionMethod(Deflate::CompressionMethod method)
{
#ifdef _WIN32
    (void)method;
#else
    dynamic_cast<BasicPngWriter*>(mWriterImpl.get())->setCompressionMethod(method);
#endif
}

void PngWriter::write(const Path& path, Image* image)
{
    mWriterImpl->write(path, image);
}
