#pragma once

#include "Pointer.h"
#include "Vector.h"

class Node;

class Edge
{
public:
    enum class State
    {
        HEALTHY,
        DIRTY
    };

    Edge(Node* node0, Node* node1, size_t id = 0);

    ~Edge();

    static Ptr<Edge> Create(Node* node0, Node* node1, size_t id = 0);

    void associateFace(size_t faceId);

    void associateWithNodes();

    void clearConnectivity();

    size_t getConnectedFaceId(size_t idx) const;

    State getState() const;

    size_t getId() const;

    size_t getNode0Id() const;

    size_t getNode1Id() const;

    Node* getNode0() const;

    Node* getNode1() const;

    size_t getNumConnectedFaces() const;

    Node* getOtherNode(Node* node) const;

    bool isOverlapping(Edge* edge) const;

    bool hasSameNodes(Edge* edge) const;

    void replaceNodes(Node* node0, Node* node1);

    void setState(State state);

    void setIndex(size_t idx);

private:
    size_t mId{0};
    Node* mNode0{nullptr};
    Node* mNode1{nullptr};

    Vector<size_t> mAssociatedFaceIds;

    State mState{State::HEALTHY};
};
