#pragma once

#include "AbstractMesh.h"

#include "String.h"
#include Map.h

class Edge;
class AbstractFace;

using EdgePtr = Ptr<Edge>;
using AbstractFacePtr = Ptr<AbstractFace>;

using VecEdges = Vector<EdgePtr>;
using VecFaces = Vector<AbstractFacePtr>;

class FaceMesh : public AbstractMesh
{
public:
    FaceMesh() = default;

    ~FaceMesh();

    virtual Ptr<AbstractMesh> copy() const = 0;

    void populate(VecNodes& nodes, VecEdges& edges, VecFaces& faces);

    Vector<size_t> getFaceNodeIds() const;

    Vector<size_t> getEdgeNodeIds() const;

    void addConstantFaceVectorAttribute(const String& tag, const Vector<double>& values);

    Vector<Vector<double> > getFaceVectorAttributes(const String& tag);

    void merge(Ptr<FaceMesh> mesh);

    const VecFaces& getFaces() const;

protected:
    void resetConnectivity();

    void resetIds();

    void replaceIfOverlapping(FaceMesh* mesh, Node* node) const;

    void replaceIfOverlapping(FaceMesh* mesh, Edge* edge) const;

    VecEdges mEdges;
    VecFaces mFaces;
};
