#include "Node.h"

Ptr<Node> Node::Create(const Point3& p, size_t index)
{
    return std::make_unique<Node>(p, index);
}

Node::~Node()
{

}

Node::Node(const Point3& p, size_t index)
    : mIndex(index),
      mPoint(p)
{

}

size_t Node::getIndex() const
{
    return mIndex;
}

void Node::updateIndex(size_t index)
{
    mIndex = index;
}

void Node::addVectorAttribute(const String& tag, const Vector<double>& values)
{
    mVectorAttributes[tag] = values;
}

Vector<double> Node::getVectorAttribute(const String& tag) const
{
    auto iter = mVectorAttributes.find(tag);
    if (iter != mVectorAttributes.end())
    {
        return iter->second;
    }
    return {};
}

const Point3& Node::getPoint() const
{
    return mPoint;
}

void Node::apply(const Transform& transform)
{
    mPoint.apply(transform);
}

bool Node::isCoincident(Node* node) const
{
    return node->getPoint() == mPoint;
}

void Node::setState(State state)
{
    mState = state;
}

Node::State Node::getState() const
{
    return mState;
}

void Node::clearConnectivity()
{
    mAssociatedEdgeIds.clear();
    mAssociatedFaceIds.clear();
}

size_t Node::getNumConnectedEdges() const
{
    return mAssociatedEdgeIds.size();
}

size_t Node::getNumConnectedFaces() const
{
    return mAssociatedFaceIds.size();
}

void Node::associateEdge(size_t edgeId)
{
    mAssociatedEdgeIds.push_back(edgeId);
}

void Node::associateFace(size_t faceId)
{
    mAssociatedFaceIds.push_back(faceId);
}

size_t Node::getConnectedEdgeId(size_t idx) const
{
    return mAssociatedEdgeIds[idx];
}

size_t Node::getConnectedFaceId(size_t idx) const
{
    return mAssociatedFaceIds[idx];
}

void Node::setIndex(size_t idx)
{
    mIndex = idx;
}


