#pragma once

#include "MaterialNode.h"

struct SceneInfo;
class SceneModel;

class GeometryNode : public MaterialNode
{
public:
    enum class Type
    {
        Path,
        Rectangle,
        Circle,
        Arc,
        Line,
        Polygon
    };

public:
    GeometryNode(const Transform& transform);
    virtual ~GeometryNode();

    virtual Type getType() = 0;

    SceneItem* getSceneItem(size_t idx) const override;
    size_t getNumSceneItems() const override;

    double getStrokeThickness() const;
    void setStrokeThickness(double thickness);

    void update(SceneInfo* sceneInfo) override;

protected:
    virtual void createOrUpdateGeometry(SceneInfo* sceneInfo) = 0;
    virtual void updateMaterial();

    double mStrokeThickness{0};
    Type mType;
    bool mGeometryIsDirty{true};

    Ptr<SceneModel> mBackgroundItem;
};

using GeometryNodePtr = Ptr<GeometryNode>;
