#include "PathNode.h"

#include "Path.h"
#include "SceneInfo.h"

#include "SceneModel.h"
#include "PostscriptReader.h"

PathNode::PathNode(const Transform& transform, const String& psPath)
	: GeometryNode(transform),
	    mPathString(psPath)
{

}

Ptr<PathNode> PathNode::Create(const Transform& transform, const String& psPath)
{
	return std::make_unique<PathNode>(transform, psPath);
}

GeometryNode::Type PathNode::getType() 
{
	return GeometryNode::Type::Path;
}

const String& PathNode::getPathString() const
{
	return mPathString;
}

void PathNode::setPathString(const String& psPath)
{
	mPathString = psPath;
}

void PathNode::createOrUpdateGeometry(SceneInfo*)
{
    auto path = std::make_unique<GeometryPath>();
    PostscriptReader reader;
    reader.read(path.get(), mPathString);

    if (!mBackgroundItem)
    {
        mBackgroundItem = std::make_unique<SceneModel>(std::move(path));
        mBackgroundItem->setName(mName + "_Model");     
    }
    else
    {
        mBackgroundItem->updateGeometry(std::move(path));
    }
}
