#pragma once

#include "SvgShapeElement.h"
#include "Point.h"

#include "XmlAttribute.h"

class SvgCircle : public SvgShapeElement
{
public:
    enum class Type
    {
        REGULAR,
        ELLIPSE
    };

    SvgCircle(Type type = Type::REGULAR);

    Point2 getLocation() const;

    double getRadius() const;

    Type getType() const;

    double getMinorRadius() const;

    void setLocation(const Point2& loc);

    void setRadius(double rad);

    void setMinorRadius(double rad);
private:
    Type mType{ Type::REGULAR };
};

class SvgRectangle : public SvgShapeElement
{
public:
    SvgRectangle();

    void setLocation(const Point2& loc);

    void setWidth(double width);

    void setHeight(double height);

    void setRadius(double radius);
};

class SvgPolygon : public SvgShapeElement
{
public:
    SvgPolygon();

    void setPoints(const Vector<Point2>& loc);
};

class SvgPolyline : public SvgShapeElement
{
public:
    SvgPolyline();

    void setPoints(const Point2& startPoint, const Vector<Point2>& loc);

    void setPoints(const Vector<Point2>& loc);
};

class SvgLine : public SvgShapeElement
{
public:
    SvgLine(const Point2& startPoint, const Point2& endPoint, size_t precision = 0);
};

class SvgPath : public SvgShapeElement
{
public:
    SvgPath();

    String getPath() const;

    void setPath(const String& mPath);
    void setFillRule(const String& fillRule);
};
