#pragma once

#include "ButtonData.h"
#include "Widget.h"
#include "Color.h"

#include "MediaResources.h"

#include <functional>
#include "String.h"

class PaintEvent;
class MouseEvent;
class TextNode;
class IconNode;

class Button : public Widget
{
public:
    using clickFunc = std::function<void(Widget* self)>;

    Button(ButtonData::Component component = ButtonData::Component::Elevated);

    ~Button();

    static Ptr<Button> Create(ButtonData::Component component = ButtonData::Component::Elevated);

    void setSvgIcon(Resource::Icon::Svg icon);

    void setLabel(const String& text);

    void setEnabled(bool isEnabled);

    void setOnClickFunction(clickFunc func);

protected:
    void onMyMouseEvent(const MouseEvent* event) override;

    bool isDirty() const override;
    void doPaint(const PaintEvent* event) override;

    void setLabelTextColor(Theme::Sys::Color color);
    void setLabelTextOpacity(float opacity);
    void setLabelTextTypescale(Theme::Sys::Typescale typescale);

    void setIconColor(Theme::Sys::Color color);
    void setIconOpacity(float opacity);

    void updateLabel(const PaintEvent* event);
    void updateIcon(const PaintEvent* event);

    void setState(ButtonData::State state);
    void resetState();
    void updateState() override;

private:
    ButtonData mStyle;

    String mLabel;
    Theme::Sys::Color mLabelTextColor;
    Theme::Sys::Typescale mLabelTextTypescale;
    float mLabelOpacity{ 1.0 };
    Ptr<TextNode> mTextNode;

    Resource::Icon::Svg mIcon{ Resource::Icon::Svg::NONE};
    Theme::Sys::Color mIconColor;
    float mIconOpacity{ 1.0 };
    Ptr<IconNode> mIconNode;

    bool mContentDirty{ true };

    clickFunc mClickFunc;
    bool mEnabled{ true };
};

using ButtonUPtr = Ptr<Button>;
