#pragma once

#include "MarkdownElement.h"
#include "MarkdownCustomElement.h"

#include "Vector.h"
#include "Pointer.h"

class MarkdownTextSpan : public MarkdownInlineElement
{
public:
    virtual ~MarkdownTextSpan() = default;

    Type getType() const override;
};


class MarkdownLink : public MarkdownInlineElement
{
public:
    MarkdownLink(const String& target);

    virtual ~MarkdownLink() = default;

    const String& getTarget() const;

    Type getType() const override;

    void setTarget(const String& target)
    {
        mTarget = target;
    }
private:
    String mTarget;
};

class MarkdownElementWithChildren : public MarkdownElement
{
public:
    void addChild(Ptr<MarkdownInlineElement> child);

    size_t getNumChildren() const;

    MarkdownInlineElement* getChild(size_t idx) const;

    MarkdownInlineElement* getLastChild() const;

    Vector<MarkdownLink*> getAllLinks() const;

private:
    Vector<Ptr<MarkdownInlineElement> > mChildren;
};

class MarkdownParagraph : public MarkdownElementWithChildren
{
public:
    virtual ~MarkdownParagraph() = default;

    Type getType() const override;

    void doFieldSubstitution(Type elementType, const String& searchPhrase, const String& replacementPhrase) override
    {
        for(auto& child : mChildren)
        {
            child->doFieldSubstitution(elementType, searchPhrase, replacementPhrase);
        }
    }
private:
    Vector<Ptr<MarkdownInlineElement> > mChildren;
};

class MarkdownBulletItem : public MarkdownElementWithChildren
{
public:
    virtual ~MarkdownBulletItem() = default;

    Type getType() const override;
};

class MarkdownBulletList : public MarkdownElement
{
public:
    virtual ~MarkdownBulletList() = default;

    Type getType() const override;

    void addChild(Ptr<MarkdownBulletItem> child);

    size_t getNumChildren() const;

    MarkdownBulletItem* getChild(size_t idx) const;

private:
    Vector<Ptr<MarkdownBulletItem> > mChildren;
};

class MarkdownHeading : public MarkdownElement
{
public:
    MarkdownHeading(unsigned level);

    virtual ~MarkdownHeading() = default;

    Type getType() const override;

    unsigned getLevel() const;

private:
    unsigned mLevel{1};
};

class MarkdownInlineQuote : public MarkdownInlineElement
{
public:

    virtual ~MarkdownInlineQuote() = default;

    Type getType() const override;
};

class MarkdownCustomInline : public MarkdownInlineElement
{
public:
    MarkdownCustomInline(MarkdownCustomElementContext* customElement);
    virtual ~MarkdownCustomInline() = default;

    Type getType() const override;

    String getTextContent() const override;
private:
    MarkdownCustomElementContext* mCustomElement{nullptr};
};

class MarkdownImage : public MarkdownInlineElement
{
public:
    MarkdownImage(const String& source, const String& alt);

    virtual ~MarkdownImage() = default;

    Type getType() const override;

    const String& getSource() const;

    const String& getAlt() const;

private:
    String mSource;
    String mAlt;
};

class MarkdownMultilineQuote : public MarkdownMultilineElement
{
public:
    MarkdownMultilineQuote(const String& tag);

    virtual ~MarkdownMultilineQuote() = default;

    Type getType() const override;
private:
    String mTag;
};

class MarkdownCustomMultiLine : public MarkdownMultilineElement
{
public:
    MarkdownCustomMultiLine(const String& tag, MarkdownCustomElementContext* customElement);

    virtual ~MarkdownCustomMultiLine() = default;

    Type getType() const override;

    String getTextContent() const override;
private:
    String mTag;
    MarkdownCustomElementContext* mCustomElement{nullptr};
};
