#pragma once

#include "Pointer.h"
#include "String.h"
#include "Vector.h"

class MarkdownDocument;
class MarkdownElement;
class MarkdownInlineElement;
class MarkdownBulletList;
class MarkdownCustomInlineElementContext;
class MarkdownCustomMultilineElementContext;

class MarkdownParser
{
public:
    MarkdownParser();

    ~MarkdownParser();

    Ptr<MarkdownDocument> run(const String& content);

    void run(const String& content, MarkdownDocument* document);

private:
    void addChildToWorkingElement(Ptr<MarkdownInlineElement> child);

    size_t checkForImage(const String& lineSection);
    size_t checkForLink(const String& lineSection);
    size_t checkForInlineQuote(const String& lineSection);
    size_t checkForCustomInline(const String& lineSection);

    bool isInMultilineBlock() const;

    void onFoundMultiLineQuote(const String& line);
    void onFoundCustomMultiLineBlock(const String& line, size_t blockSlot);
    void onFoundHeading(const String& line);
    void onFoundBulletItem(const String& line);

    void onEmptyLine();
    void onSectionFinished();
    void onTextSpanFinished();

    void processLine(const String& line);

    bool startsWithMultiLineQuote(const String& line) const;
    int startsWithCustomMultilineBlock(const String& line) const;
    bool startsWithHeading(const String& line) const;
    bool startsWithBulletItem(const String& line) const;

    size_t mCustomDelimiterIndex{0};

    MarkdownElement* mWorkingElement{nullptr};
    MarkdownBulletList* mWorkingBulletList{nullptr};

    MarkdownInlineElement* mWorkingTextSpan{nullptr};
    String mWorkingLine;

    MarkdownDocument* mWorkingDocument{nullptr};
};
